==> kinescope-1.1.x-dev.tar.gz: kinescope/src/Plugin/Field/FieldWidget/KinescopeWidget.php <==
<?php

namespace Drupal\kinescope\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Utility\NestedArray;

/**
 * Defines the 'Kinescope' field widget.
 *
 * @FieldWidget(
 *   id = "kinescope_widget",
 *   label = @Translation("Kinescope widget"),
 *   field_types = {"kinescope"},
 * )
 */
class KinescopeWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element['video_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Kinescope URL'),
      '#description' => $this->t('Paste here the Kinescope URL.'),
      '#default_value' => $items[$delta]->video_url ?? '',
      '#required' => TRUE,
      '#element_validate' => [[get_class($this), 'validateUrl']],
    ];
    $element['video_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Kinescope ID'),
      '#disabled' => TRUE,
      '#default_value' => $items[$delta]->video_id ?? '',
    ];
    $element['is_responsive'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Is responsive?'),
      '#default_value' => $items[$delta]->is_responsive ?? TRUE,
    ];
    $element['width'] = [
      '#type' => 'number',
      '#title' => $this->t('Width'),
      '#description' => $this->t('Width of the video player. Non-affected if responsive is checked.'),
      '#default_value' => $items[$delta]->width ?? 640,
      '#min' => 1,
      '#max' => 65535,
    ];
    $element['height'] = [
      '#type' => 'number',
      '#title' => $this->t('Height'),
      '#description' => $this->t('Height of the video player. Non-affected if responsive is checked.'),
      '#default_value' => $items[$delta]->height ?? 480,
      '#min' => 1,
      '#max' => 65535,
    ];
    $element['options'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Options'),
      '#description' => $this->t('Check options here: https://docs.kinescope.io/player/latest/iframe/IframeRegular.html'),
      '#default_value' => $items[$delta]->options ?? 'autoplay; fullscreen; picture-in-picture; encrypted-media; gyroscope; accelerometer; clipboard-write; screen-wake-lock;',
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function validateUrl(array $element, FormStateInterface $form_state, $form) {
    $video_url = $element['#value'];
    if (empty($video_url)) {
      $form_state->setErrorByName('video_url', t('The Kinescope URL cannot be empty.'));
    }
    elseif (!preg_match('/^https?:\/\/kinescope\.io\/[a-zA-Z0-9_-]+$/', $video_url)) {
      $form_state->setErrorByName('video_url', t('The Kinescope URL is not valid.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    foreach ($values as &$value) {
      if (!empty($value['video_url']) && preg_match('/kinescope\.io\/([a-zA-Z0-9_-]+)/', $value['video_url'], $matches)) {
        $value['video_id'] = $matches[1];
      }
    }
    return $values;
  }

}

==> kinescope-1.2.x-dev.tar.gz: kinescope/src/Plugin/Field/FieldWidget/KinescopeWidget.php <==
<?php

namespace Drupal\kinescope\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\devel\Plugin\Devel\Dumper\Kint;
use Drupal\kinescope\Plugin\media\Source\Kinescope;

/**
 * Defines the 'Kinescope' field widget.
 *
 * @FieldWidget(
 *   id = "kinescope_widget",
 *   label = @Translation("Kinescope widget"),
 *   field_types = {"kinescope"},
 * )
 */
class KinescopeWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element['video_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Kinescope URL'),
      '#description' => $this->t('Paste here the Kinescope URL.'),
      '#default_value' => $items[$delta]->video_url ?? '',
      '#required' => TRUE,
      '#element_validate' => [[get_class($this), 'validateUrl']],
    ];
    $element['video_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Kinescope ID'),
      '#disabled' => TRUE,
      '#default_value' => $items[$delta]->video_id ?? '',
    ];
    $element['is_responsive'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Is responsive?'),
      '#default_value' => $items[$delta]->is_responsive ?? TRUE,
    ];
    $element['width'] = [
      '#type' => 'number',
      '#title' => $this->t('Width'),
      '#description' => $this->t('Width of the video player. Non-affected if responsive is checked.'),
      '#default_value' => $items[$delta]->width ?? 640,
      '#min' => 1,
      '#max' => 65535,
    ];
    $element['height'] = [
      '#type' => 'number',
      '#title' => $this->t('Height'),
      '#description' => $this->t('Height of the video player. Non-affected if responsive is checked.'),
      '#default_value' => $items[$delta]->height ?? 480,
      '#min' => 1,
      '#max' => 65535,
    ];
    $element['options'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Options'),
      '#description' => $this->t('Check options here: https://docs.kinescope.io/player/latest/iframe/IframeRegular.html'),
      '#default_value' => $items[$delta]->options ?? 'autoplay; fullscreen; picture-in-picture; encrypted-media; gyroscope; accelerometer; clipboard-write; screen-wake-lock;',
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function validateUrl(array $element, FormStateInterface $form_state, $form) {
    $video_url = $element['#value'];
    if (empty($video_url)) {
      $form_state->setErrorByName('video_url', t('The Kinescope URL cannot be empty.'));
    }
    elseif (!Kinescope::validateUrl($video_url)) {
      $form_state->setErrorByName('video_url', t('The Kinescope URL is not valid.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    foreach ($values as &$value) {
      if (!empty($value['video_url'])) {
        $video_id = Kinescope::getVideoId($value['video_url'] ?? '');
        $value['video_id'] = $video_id;
      }
    }
    return $values;
  }

}
